package speiger.src.coreengine.utils.helpers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.file.Files;
import java.nio.file.Path;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

public class JsonUtil
{
	public static JsonObject loadFile(Path path)
	{
		try(BufferedReader reader = Files.newBufferedReader(path))
		{
			return JsonParser.parseReader(reader).getAsJsonObject();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return new JsonObject();
		}
	}
	
	public static JsonObject loadFile(File file)
	{
		try(Reader reader = new BufferedReader(new FileReader(file)))
		{
			return JsonParser.parseReader(reader).getAsJsonObject();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return new JsonObject();
		}
	}
	
	public static JsonObject loadFile(InputStream stream)
	{
		try(Reader reader = new BufferedReader(new InputStreamReader(stream)))
		{
			return JsonParser.parseReader(reader).getAsJsonObject();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return new JsonObject();
		}
	}
	
	public static boolean getOrDefault(JsonObject obj, String name, boolean defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsBoolean();
	}
	
	public static byte getOrDefault(JsonObject obj, String name, byte defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsByte();
	}
	
	public static short getOrDefault(JsonObject obj, String name, short defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsShort();
	}
	
	public static int getOrDefault(JsonObject obj, String name, int defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsInt();
	}
	
	public static long getOrDefault(JsonObject obj, String name, long defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsLong();
	}
	
	public static float getOrDefault(JsonObject obj, String name, float defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsFloat();
	}
	
	public static double getOrDefault(JsonObject obj, String name, double defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsDouble();
	}
	
	public static String getOrDefault(JsonObject obj, String name, String defaultValue)
	{
		JsonElement el = obj.get(name);
		return el == null ? defaultValue : el.getAsString();
	}
	
	public static JsonArray toArray(byte[] values)
	{
		JsonArray array = new JsonArray();
		for(int i = 0,m=values.length;i<m;i++)
		{
			array.add(values[i]);
		}
		return array;
	}
	
	public static byte[] parseArray(JsonArray array)
	{
		byte[] data = new byte[array.size()];
		for(int i = 0,m=data.length;i<m;i++)
		{
			data[i] = array.get(i).getAsByte();
		}
		return data;
	}
}
